@@ -441,7 +441,7 @@ class FeedManager(gobject.GObject):
                     # Get the next 20 items
                     continuation = nodes[0].firstChild.data
                     req = urllib2.Request(feed_request+'?c='+continuation, headers = auth_headers)
-                    r = urllib2.urlopen(req)
+                    feed_response = urllib2.urlopen(req)
                     response_data = feed_response.read()
             except:
                 return False
