@@ -29,7 +29,6 @@ from utils import *
 from webkit import WebView
 import calendar
 import constants
-import dbus
 import feedparser
 import gettext
 import glib
@@ -1178,35 +1177,6 @@ class EntriesView(hildon.GtkTreeView):
             + '</span>\n<span size="small">' \
             + gobject.markup_escape_text(unescape(title)) + '</span>'
 
-    def get_visual_entry_date(self, entry):
-        if entry.has_key('updated_parsed') == False:
-            if entry.has_key('updated'):
-                return '<span size="xx-small">%s</span>' % \
-                    entry.updated
-            else:
-                return ''
-
-        now = time.localtime()
-
-        if now.tm_year == entry.updated_parsed.tm_year:
-            # Today
-            if now.tm_yday == entry.updated_parsed.tm_yday:
-                return '<span size="xx-small">%s</span>' % \
-                    time.strftime('%H:%M', entry.updated_parsed)
-            # This week
-            elif now.tm_yday - entry.updated_parsed.tm_yday < 7 and \
-                    entry.updated_parsed.tm_wday < now.tm_wday:
-                return '<span size="xx-small">%s</span>' % \
-                    time.strftime('%a', entry.updated_parsed)
-            # This year
-            else:
-                return '<span size="xx-small">%s</span>' % \
-                    time.strftime('%d %b', entry.updated_parsed)
-        # Anything else
-        else:
-            return '<span size="xx-small">%s</span>' % \
-                time.strftime('%x', entry.updated_parsed)
-
 class EntriesWindow(hildon.StackableWindow):
 
     ENTRIES_MODEL_TEXT_COLUMN = 1
@@ -1307,7 +1277,7 @@ class EntriesWindow(hildon.StackableWindow):
             model.set(entry_iter,
                       self.ENTRIES_MODEL_TEXT_COLUMN, self.view.get_visual_entry_text(entry),
                       self.ENTRIES_MODEL_DATA_COLUMN, entry,
-                      self.ENTRIES_MODEL_DATE_COLUMN, self.view.get_visual_entry_date(entry),
+                      self.ENTRIES_MODEL_DATE_COLUMN, get_visual_entry_date(entry, True),
                       self.ENTRIES_MODEL_DATE_PARSED_COLUMN, calendar.timegm(date) if date else 0)
 
     def _on_connection_changed(self, conn_manager):
@@ -1572,54 +1542,27 @@ class EntriesWindow(hildon.StackableWindow):
         else:
             self.align.set_padding(4, 0 , 4, 4)
 
-class ItemWindowHeader(gtk.HBox):
+class ItemWindowLandscapeToolbar(gtk.VBox):
 
     def __init__(self, homogeneous, spacing):
-        super(ItemWindowHeader, self).__init__(homogeneous, spacing)
-
-        # Header
-        header_vbox = gtk.VBox()
-        navigation_hbox = gtk.HBox(False, 0)
-
-        # Author and URL
-        self.title = gtk.Label('')
-        self.title.set_alignment(0,0.5)
-        header_vbox.pack_start(self.title, True, False)
-        self.title.show()
-        header_vbox.show()
+        super(ItemWindowLandscapeToolbar, self).__init__(homogeneous, spacing)
 
         # Navigation buttons. Sizes are important. We set the size of the icon but
         # we let the button grow as needed
-        self.button_up = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.button_up = gtk.Button()
         button_up_img = gtk.Image()
         button_up_img.set_from_icon_name(constants.ICON_UP_NAME,
                                               gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.button_up.set_image(button_up_img)
-        self.button_down = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
+        self.button_down = gtk.Button()
         button_down_img = gtk.Image()
         button_down_img.set_from_icon_name(constants.ICON_DOWN_NAME,
                                                 gtk.HILDON_SIZE_FINGER_HEIGHT)
         self.button_down.set_image(button_down_img)
 
-        navigation_hbox.pack_start(self.button_up, False, False, 0)
-        navigation_hbox.pack_start(self.button_down, False, False, 0)
-        navigation_hbox.show_all()
-
-        self.pack_start(header_vbox, True, True)
-        self.pack_start(navigation_hbox, False, False)
-
-        # Signals
-        #self.url.connect('clicked', self._link_button_clicked)
-
-    def _link_button_clicked(self, link_button):
-        print 'Open ' + link_button.get_uri()
-
-    def set_item(self, item):
-
-        self.item = item
-
-        self.title.set_markup('<span size="small">' + glib.markup_escape_text(unescape(item.title)) + '</span>')
-        #self.url.set_uri(item.link)
+        self.pack_start(self.button_up, True, True, 0)
+        self.pack_start(self.button_down, True, True, 0)
+        self.show_all()
 
 class ItemWindow(hildon.StackableWindow):
     __gsignals__ = {
@@ -1632,27 +1575,29 @@ class ItemWindow(hildon.StackableWindow):
         self.manager = manager
         self.settings = settings
         self.feed_data = feed_data
+        self._item = None
         self._conn_manager = conn_manager
         self._conn_manager.connect('connection-changed', self._on_connection_changed)
 
         self._osso_context = osso.Context(constants.RSS_COMPACT_NAME, constants.RSS_VERSION, False)
         self._screen_device = osso.DeviceState(self._osso_context)
-
-        # For opening links in browser
-        self._browser_dbus_iface = None
+        self._osso_rpc = osso.Rpc(self._osso_context)
 
         self._prev_item_reference = None
         self._next_item_reference = None
 
         # Header
-        self.item_window_header = ItemWindowHeader(False, 16)
-        self.item_window_header.button_up.connect("clicked", self._up_button_clicked)
-        self.item_window_header.button_down.connect("clicked", self._down_button_clicked)
+        self.landscape_toolbar = ItemWindowLandscapeToolbar(True, 2)
+        self.landscape_toolbar.button_up.connect("clicked", self._up_button_clicked)
+        self.landscape_toolbar.button_down.connect("clicked", self._down_button_clicked)
 
         # HTML renderer
         self.view = WebView()
         self.view.set_full_content_zoom(True)
 
+        # Disable navigation inside ReSiStance. Open links in browser
+        self.view.connect('navigation_requested', self._navigation_requested_cb)
+
         # Disable text selection
         self.view.connect("motion-notify-event", lambda w, ev: True)
 
@@ -1669,20 +1614,14 @@ class ItemWindow(hildon.StackableWindow):
         self.pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
         self.view.show()
 
-        # Header Alignment
-        header_align = gtk.Alignment(0, 0, 1, 1)
-        header_align.set_padding(4, 0 , 16, 16)
-        header_align.add(self.item_window_header)
-        header_align.show()
-
-        vbox = gtk.VBox(False, 8)
-        vbox.pack_start(header_align, False, False)
-        vbox.pack_start(self.pannable, True, True)
-        self.item_window_header.show()
+        hbox = gtk.HBox(False, 4)
+        hbox.pack_start(self.pannable, True, True)
+        hbox.pack_start(self.landscape_toolbar, False, False)
+        self.landscape_toolbar.show()
         self.pannable.show()
 
-        self.add(vbox)
-        vbox.show()
+        self.add(hbox)
+        hbox.show()
 
         self._create_menu()
         self._create_toolbar()
@@ -1691,9 +1630,9 @@ class ItemWindow(hildon.StackableWindow):
         self.connect('configure-event', self._on_configuration_changed)
 
     def _on_connection_changed(self, conn_manager):
-        if conn_manager.is_online():
-            if 'enclosures' in self.item_window_header.item:
-                self._download_button.show()
+        if conn_manager.is_online() and 'enclosures' in self._item \
+                and len(self._item.enclosures) > 0:
+            self._download_button.show()
         else:
             self._download_button.hide()
 
@@ -1702,12 +1641,11 @@ class ItemWindow(hildon.StackableWindow):
         # Show toolbar in portrait mode. Show header in landscape mode
         if event.width > event.height:
             self.pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
-            self.item_window_header.show_all()
+            self.landscape_toolbar.show_all()
             self.toolbar.hide_all()
         else:
             self.pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_VERT)
-            self.item_window_header.button_up.hide()
-            self.item_window_header.button_down.hide()
+            self.landscape_toolbar.hide()
             self.toolbar.show_all()
 
     def _create_toolbar(self):
@@ -1759,12 +1697,6 @@ class ItemWindow(hildon.StackableWindow):
         button.show()
         menu.append(button)
 
-        button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
-        button.set_label(_('View in Browser'))
-        button.connect('clicked', self._view_in_browser_cb)
-        button.show()
-        menu.append(button)
-
         self._download_button = hildon.GtkButton(gtk.HILDON_SIZE_FINGER_HEIGHT)
         self._download_button.set_label(_('Download'))
         self._download_button.connect('clicked', self._download_item_cb)
@@ -1794,17 +1726,6 @@ class ItemWindow(hildon.StackableWindow):
         value_label.set_alignment(0.0, 0.5)
         table.attach(value_label, 1, 2, y_coord, y_coord + 1, gtk.FILL, 0)
 
-    def _view_in_browser_cb(self, button):
-        item = self.item_window_header.item
-        if 'link' in item:
-            # Lazy load. We do not do this at window creation time for
-            # performance reasons
-            if not self._browser_dbus_iface:
-                bus = dbus.SystemBus()
-                proxy_browser = bus.get_object('com.nokia.osso_browser','/com/nokia/osso_browser/request')
-                self._browser_dbus_iface = dbus.Interface(proxy_browser, 'com.nokia.osso_browser')
-            self._browser_dbus_iface.load_url(item.link)
-
     def _item_details_cb(self, button):
         dialog = gtk.Dialog()
         dialog.set_title(_('Item Details'))
@@ -1813,7 +1734,7 @@ class ItemWindow(hildon.StackableWindow):
         pannable.set_size_request_policy(hildon.SIZE_REQUEST_CHILDREN)
         pannable.set_property("mov-mode", hildon.MOVEMENT_MODE_VERT)
 
-        item = self.item_window_header.item
+        item = self._item
 
         table = gtk.Table()
         table.set_row_spacings(6)
@@ -1881,6 +1802,8 @@ class ItemWindow(hildon.StackableWindow):
 
     def _set_item(self, item, path):
 
+        self._item = item
+
         # Update next&prev references
         self._update_next_prev_references(path)
 
@@ -1889,9 +1812,6 @@ class ItemWindow(hildon.StackableWindow):
             author = self.feed_data.feed.title
         self.set_title(unescape(author))
 
-        # Set item in header
-        self.item_window_header.set_item(item)
-
         # Get content type and body
         content_type = 'text/html'
         if 'content' in item:
@@ -1911,19 +1831,25 @@ class ItemWindow(hildon.StackableWindow):
             # Should never happen
             body = _('No text')
 
-        self.view.load_string(body.encode('utf-8'), content_type, 'utf-8', '')
+        # Feed header
+        header = '<font size="5"><a href=%s>%s</a></font><br><a href=%s>%s</a> ' % \
+            (item.link, item.title, self.feed_data.feed.link, self.feed_data.feed.title)
+        # Date is treated separatedely for translation purpouses
+        date = _('on %s') % get_visual_entry_date(item,False) if 'updated' in item else ''
+
+        text = ''.join((header, date, '<hr>', body))
 
         # Write HTML
         try:
-            self.view.load_string(body.encode(self.feed_data.encoding), content_type, self.feed_data.encoding, '')
+            self.view.load_string(text.encode(self.feed_data.encoding), content_type, self.feed_data.encoding, '')
         except UnicodeEncodeError:
             # In case of decoding error then try with UTF-8
             try:
-                self.view.load_string(body.encode('utf-8'), content_type, 'utf-8', '')
+                self.view.load_string(text.encode('utf-8'), content_type, 'utf-8', '')
             except UnicodeEncodeError:
                 # Ok, let's fallback to ASCII then...
                 try:
-                    self.view.load_string(body.encode('us-ascii'), content_type, 'us-ascii', '')
+                    self.view.load_string(text.encode('us-ascii'), content_type, 'us-ascii', '')
                 except UnicodeEncodeError:
                     # If everything else fails, then clear the view and show an error
                     self.view.load_string('', content_type, 'ascii', '')
@@ -1934,9 +1860,9 @@ class ItemWindow(hildon.StackableWindow):
         prev_item_missing = not self._prev_item_reference or not self._prev_item_reference.valid()
         next_item_missing = not self._next_item_reference or not self._next_item_reference.valid()
 
-        self.item_window_header.button_up.set_sensitive(not prev_item_missing)
+        self.landscape_toolbar.button_up.set_sensitive(not prev_item_missing)
         toolbar_button_list[0].set_sensitive(not prev_item_missing)
-        self.item_window_header.button_down.set_sensitive(not next_item_missing)
+        self.landscape_toolbar.button_down.set_sensitive(not next_item_missing)
         toolbar_button_list[1].set_sensitive(not next_item_missing)
 
         # Mark item as read
@@ -1949,9 +1875,17 @@ class ItemWindow(hildon.StackableWindow):
         # Keep the display on for 60" without user input.
         self._screen_device.display_blanking_pause()
 
+    def _navigation_requested_cb(self, web_view, web_frame, navigation_request):
+        print 'loading ' + navigation_request.get_uri()
+        # Open link in a web browser
+        self._osso_rpc.rpc_run_with_defaults("osso_browser", "open_new_window",
+                                             (navigation_request.get_uri(),))
+        # Just deny navigation. Note that there seem not to be any
+        # define for data and we have to use an ugly plain 1
+        return 1
+
     def _download_item_cb(self, button):
-        item = self.item_window_header.item
-        url_file = item.enclosures[0]['href']
+        url_file = self._item.enclosures[0]['href']
         file = os.path.basename(urllib.url2pathname(url_file))
         folder = self.settings.auto_download_folder
 
